#include "alpm/manager.h"
#include "alpm/utilities.h"
#include "alpm/config.h"
#include "alpm/resolvebuildorder.h"
#include "alpm/mingwbundle.h"

#include "network/server.h"

#include <c++utilities/application/argumentparser.h>
#include <c++utilities/application/failure.h>

#include <QCoreApplication>

#include <iostream>
#include <algorithm>

using namespace std;
using namespace ApplicationUtilities;
using namespace PackageManagement;
using namespace Network;

int main(int argc, char *argv[])
{
    // setup the argument parser
    ArgumentParser parser;
    ConfigArgs configArgs(parser);
    parser.setIgnoreUnknownArguments(false);
    // parse command line arguments
    try {
        parser.parseArgs(argc, argv);
    } catch (Failure &e) {
        cerr << "Unable to parse arguments: " << e.what() << endl;
        return 2;
    }
    try {
        // load configuration
        Config config;
        config.loadFromConfigFile(configArgs);
        config.loadFromArgs(configArgs);
        if(find_if(parser.mainArguments().cbegin(), parser.mainArguments().cend(), [&configArgs] (const Argument *arg) {
                   return arg != &configArgs.helpArg && arg->isPresent();
                }) != parser.mainArguments().cend()) {
            cerr << "Loading databases ..." << endl;
            // create app
            QCoreApplication application(argc, argv);
            // setup ALPM
            Manager manager(config);
            manager.applyPacmanConfig();
            manager.applyRepoIndexConfig();
            if(configArgs.serverArg.isPresent()) {
                // setup the server
                Server server(manager, manager.config());
                QObject::connect(&server, &Server::closed, &application, &QCoreApplication::quit);
                // run Qt loop
                return application.exec();
            } else if(configArgs.buildOrderArg.isPresent()) {
                BuildOrderResolver resolver(manager);
                const QStringList results = resolver.resolve(configArgs.buildOrderArg.values());
                // print results
                cout << "Results: ";
                for(const auto &pkgName : results) {
                    cout << pkgName.toLocal8Bit().data() << ' ';
                }
                cout << endl;
            } else if(configArgs.mingwBundleArg.isPresent()) {
                MingwBundle bundle(manager, configArgs.mingwBundleArg.values(), configArgs.iconThemesArg.values());
                bundle.createBundle(configArgs.targetDirArg.isPresent() ? configArgs.targetDirArg.values().front() : string("."),
                                    configArgs.targetNameArg.values().front(),
                                    configArgs.targetFormatArg.isPresent() ? configArgs.targetFormatArg.values().front() : string("zip"));
            }
        } else {
            cout << "No command line arguments specified. See --help for available commands." << endl;
        }
    } catch (std::exception &e) {
        cerr << "Error: " << e.what() << endl;
        return 1;
    }
}
